/***************************************************************************
                          palettecfg.cpp  -  description
                             -------------------
    begin                : lun abr 15 2002
    Copyright	Giuseppe Della Bianca<giusdbg@gmail.com>

    original author	 : Miguel Novas
    email                : michaell@teleline.es
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qlabel.h>
#include <qpalette.h>
#include <qcombobox.h>
#include <qdebug.h>

#include <kcolorbutton.h>

#include "general.h"
#include "panelswindow.h"
#include "palettecfg.h"


PaletteCfg::PaletteCfg( QWidget *parent, const QString &name ): PaletteCfgDesign(parent,name)
{
    shortName.clear();
    longName.clear();
    subName.clear();

 slotComboSelected(0);
 connect( comboSelect,SIGNAL(activated(int)),this,SLOT(slotComboSelected(int)) );
 connect( buttonColor,SIGNAL(changed(const QColor &)),this,SLOT(slotColorChanged(const QColor &)) );
 connect( buttonDefaultPalette, SIGNAL(clicked()),this,SLOT(slotPaletteDefault()) );
}

PaletteCfg::~PaletteCfg()
{
}

void PaletteCfg::slotComboSelected(int index)
{
 QPalette colors= framePanel->palette() /* .color( QPalette::Active, QPalette::Window ) */ ;
 QColor color;
 switch(index) {
  case 0: color= colors.window().color(); break;
  case 1: color= colors.dark().color();       break;
  case 2: color= colors.text().color();       break;
  case 3: color= colors.windowText().color(); break;
 }
 buttonColor->blockSignals(true);
 buttonColor->setColor(color);
 buttonColor->blockSignals(false);
}

void PaletteCfg::slotColorChanged(const QColor &newColor)
{
 QPalette pal= framePanel->palette();
 switch(comboSelect->currentIndex()) {
  case 0: pal.setColor(QPalette::Window,newColor);
          pal.setColor(QPalette::Light     ,newColor); break;
  case 1: pal.setColor(QPalette::Dark      ,newColor); break;
  case 2: pal.setColor(QPalette::Text      ,newColor); break;
  case 3: pal.setColor(QPalette::WindowText,newColor); break;
  default: return;
 }
 setPanelPalette(pal,false);
 palDefault= false;
}

void PaletteCfg::slotPaletteDefault()
{
 palDefault= true;
 QPalette pal;
 if( !longName.isEmpty() ) {
   getPalette(pal, qsFL1( "General" ), true);
 } else {
   getDefaultPalette(pal);
 }
 setPanelPalette(pal,true);
}

void PaletteCfg::setPanelPalette(const QPalette &pal, bool updateColorButton)
{
  QPalette palette;
  
 framePanel->setPalette(pal); 
  palette.setColor(labelTitle->foregroundRole(), pal.color ( QPalette::Text ) );
  labelTitle->setPalette( palette );
 
 if(updateColorButton) slotComboSelected(comboSelect->currentIndex());
}

/* warning, is sensor and not device sensor */
void PaletteCfg::readPalette( const QString sensorShortName, const QString sensorLongName, const QString sensorSubName )
{
    Q_UNUSED ( sensorShortName )
    Q_UNUSED ( sensorSubName )

//  qDebug() << "\033[91m" << "qDebug(): PaletteCfg::readPalette() " << sensorLongName <<"\033[0m";

    longName = sensorLongName;

 QPalette pal;
 palDefault= !getPalette(pal,sensorLongName,false);
 if(palDefault) getPalette(pal, qsFL1( "General" ), true);
 setPanelPalette(pal,true);
}

void PaletteCfg::savePalette()
{
//  qDebug() << "\033[91m" << "qDebug(): PaletteCfg::savePalette() " << groupName <<"\033[0m";

//  setGroupConf ( !groupName.isEmpty() ? groupName : "General" );
    if ( longName.isEmpty() )
        setGroupConf ( qsFL1( "General" ) );
    else
        setSensorDevGroupConf ( shortName, longName, subName );

    if ( palDefault )
        setEntryConf ( qsFL1( "PaletteActive" ), false );
    else
    {
        QPalette qpColors = framePanel->palette() /* .color ( QPalette::Window ) */ ;

        setEntryConf ( qsFL1( "PaletteActive" ), true );
        setEntryConf ( qsFL1( "ColorBackground" ), qpColors.window().color() );
        setEntryConf ( qsFL1( "ColorShadow" ), qpColors.dark().color()       );
        setEntryConf ( qsFL1( "ColorValue" ), qpColors.windowText().color() );
        setEntryConf ( qsFL1( "ColorTitle" ), qpColors.text().color()       );      
    }
}
