#ifndef CHART_H
#define CHART_H

#include <qstring.h>
#include <qvariant.h>
#include <qlabel.h>

#include <qmainwindow.h>
#include <qwidget.h>

#include <qchart.h>
#include <qchartview.h>
#include <QtCharts/QAbstractAxis>
#include <QtCharts/QSplineSeries>
#include <QtCharts/QValueAxis>
#include <QtCore/QRandomGenerator>
#include <QtCore/QTimer>

#include "general.h"


#if (QT_VERSION < QT_VERSION_CHECK(6, 0, 0))
using namespace QtCharts;
#endif

class Chart;

class QLineChart : public QWidget
{
    Q_OBJECT
public:

    struct T_ParData
    {
        bool enableValue1;
        bool enableValue2;
        bool enableValue3;
        bool enableValue4;
        bool enableDescr1;
        bool enableDescr2;

        QVariant qvValue1;
        QVariant qvValue2;
        QVariant qvValue3;
        QVariant qvValue4;
        QVariant qvDescr1;
        QVariant qvDescr2;

        void init( bool enableValue )
        {
            enableValue1 = enableValue;
            enableValue2 = enableValue;
            enableValue3 = enableValue;
            enableValue4 = enableValue;
            enableDescr1 = enableValue;
            enableDescr2 = enableValue;

            qvValue1.clear();
            qvValue2.clear();
            qvValue3.clear();
            qvValue4.clear();
            qvDescr1.clear();
            qvDescr2.clear();
        }
    };
    
    struct T_ValData
    {
        bool visibleValue1;
        bool visibleValue2;
        bool visibleValue3;
        bool visibleValue4;
        bool visibleDescr1;
        bool visibleDescr2;

        QString qsValue1;
        QString qsValue2;
        QString qsValue3;
        QString qsValue4;
        QString qsDescr1;
        QString qsDescr2;

        QColor qcValue1;
        QColor qcValue2;
        QColor qcValue3;
        QColor qcValue4;
        QColor qcDescr1;
        QColor qcDescr2;
        
        void init()
        {
            visibleValue1 = false;
            visibleValue2 = false;
            visibleValue3 = false;
            visibleValue4 = false;
            visibleDescr1 = false;
            visibleDescr2 = false;

            qsValue1 = qsFL1 ( "" );
            qsValue2 = qsFL1 ( "" );
            qsValue3 = qsFL1 ( "" );
            qsValue4 = qsFL1 ( "" );
            qsDescr1 = qsFL1 ( "" );
            qsDescr2 = qsFL1 ( "" );
            
            qcValue1 = QColor();
            qcValue2 = QColor();
            qcValue3 = QColor();
            qcValue4 = QColor();
            qcDescr1 = QColor();
            qcDescr2 = QColor();
        }
    };

    struct T_ChartData
    {
        qreal qrOldMaxValue;
        qreal qrOldMinValue;
        qreal qrMaxValue;
        qreal qrMinValue;

        QList<QPointF> dataList;

        void init()
        {
            qrOldMaxValue = qrealMIN;
            qrOldMinValue = qrealMAX;
            qrMaxValue = qrealMAX;
            qrMinValue = qrealMIN;

            dataList.clear();
        }
    };


    QLineChart ( QWidget *parent );
    ~QLineChart();

    void setValue( struct T_ParData &parData );
    void setValueVisible( struct T_ParData &parData );
    void setValueColor( struct T_ParData &parData );

    void addValue( struct T_ParData &parData, qreal qrXAxisStepIn );

    bool getValueVisible() { return ( valChart.visibleValue1 ); };

protected:
    void resizeEvent( QResizeEvent *qreEvent ) override;

private:
    QPalette getPaletteColor( QColor qcColor );
    void addValue( struct T_ChartData &chartData, QString qsYAxisValueIn );
    void setMinMaxValue( struct T_ChartData &chartAllData, struct T_ChartData &chartData, QPointF *qpfValue );
    void setMinMaxYAxis( struct T_ChartData &chartData, QValueAxis *qvaValue );
    void updateSeries( struct T_ChartData &chartData, QAbstractSeries *series );
    void update();

    struct T_ValData valChart;

    QLabel *qlValue1;
    QLabel *qlValue2;
    QLabel *qlValue3;
    QLabel *qlValue4;
    QLabel *qlDescr1;
    QLabel *qlDescr2;

    int tickYAxis;
    int tickXAxis;
    qreal qrXAxisStep;

    QChart qcChart;
    QChartView *qcvView;
    QLineSeries *qlsSeries1;
    QLineSeries *qlsSeries2;
    QLineSeries *qlsSeries3;
    QLineSeries *qlsSeries4;
    QValueAxis qvaXAxis;
    QValueAxis qvaYAxis;

    struct T_ChartData chartAllData;
    
    struct T_ChartData chartData1;
    struct T_ChartData chartData2;
    struct T_ChartData chartData3;
    struct T_ChartData chartData4;
};


class testChart : public QWidget
{
   Q_OBJECT
public:
    testChart();
    ~testChart();
    
    QLineChart *qclChart;
    QLineChart *qclChart2;

    QWidget *qwChart;
    QWidget *qwChart2;
    QFrame *qfChart;
    QFrame *qfChart2;

private:

    void timerEvent( QTimerEvent * ) override;

    int idTimer;
    
    qreal testStepY;
    qreal testStepX;
    qreal testPosY;    
};

#endif
