/***************************************************************************
    file:           sensorpanel.cpp
    description:    sensors panel widget
    begin:          09/06/2024
    Copyright:      Giuseppe Della Bianca<giusdbg@gmail.com>
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qvariant.h>
#include <qevent.h>

#include "general.h"
#include "sensordev.h"
#include "sensorpanel.h"

QColor SensorPanel::color4 = QColor( Qt::red );
QColor SensorPanel::color3 = QColor( 200, 0, 0 );
QColor SensorPanel::color2 = QColor( 128, 32, 0 );
QColor SensorPanel::color1 = QColor( /* 0, 220, 0 */ Qt::darkGreen );

SensorPanel::SensorPanel( PanelsWindow *panelsWindowIn, Sensor *newSensor, const QString &name ) : Panel( /* parent */ nullptr, name )
{  
  idTimer = 0;
  
 alarm= false;
 //
    sensor = newSensor;
    panelsWindow = panelsWindowIn;

    qlChart = nullptr;

//  setGroupConf ( sensor->objectName() );
    setSensorDevGroupConf ( sensor->getShortName(), sensor->getLongName(), sensor->getSubName() );
//    qsShowType = getEntryConf ( qsFL1( "show" ), qsFL1( "false" ), ksensorsSetting::AutoDefault );

    if ( panelsWindow->getMeterTypes() & MeterTypes::Chart )
        qlChart = new QLineChart ( this );

 //
 createTitleWidget();
 createGraphicWidget();
 createValueWidget();
 //
 slotUpdateConfig();
 
 show();
 setParent( panelsWindowIn );

 //
// connect( sensor, SIGNAL(valueChanged()) , this, SLOT(setValue()) );
 connect( sensor, SIGNAL(panelDockConfigChanged()), this, SLOT(slotUpdateConfig()) );
 //
}

SensorPanel::~SensorPanel()
{
}

void SensorPanel::slotUpdateConfig()
{
    vMask.init();
    vMask.width = 0;
    vMask.prec = 0;
    vMask.formType = 'f';
    
    switch(sensor->getType())
    {
   case Sensor::lmTemp:
     if(arc) {
       arc->setValueRanges(sensor->celsiusTo(0),sensor->celsiusTo(100));
       arc->setValueMinLimit( sensor->getValueMin(),0);
       arc->setValueMaxLimit( sensor->getValueMax(),15);
       arc->setColorRanges(Qt::red,Qt::green,Qt::red);
     }
     
     switch( sensor->getSensorDev()->getTempScale() )
     {
     case Sensor::dgKelvin:
        vMask.postTxt = qsFL1( "K" );

        break;
      default:
        vMask.postTxt = qsFUF8( ( qlChart ? "°" : "^" ) );

        break;
     }

     break;
   case Sensor::lmFan :
     if(arc) {
       arc->setValueRanges(0,10000);
       arc->setValueMinLimit( sensor->getValueMin(),15);
       arc->setValueMaxLimit( sensor->getValueMax(),0);
       arc->setColorRanges(Qt::red,Qt::green,Qt::green);
     }
     
     vMask.postTxt = qsFL1( " RPM" );
     
     break;

   case Sensor::lmVoltage :
     if(arc) {
       double min= sensor->getValueMin();
       double max= sensor->getValueMax();
       arc->setValueRanges( min - (max-min)/3, max + (max-min)/3 );
       arc->setValueMinLimit( min,10);
       arc->setValueMaxLimit( max,10);
       arc->setColorRanges(Qt::blue,QColor(0,200,150),Qt::blue);
       arc->setArrowColor(Qt::darkBlue);
     }
     
     vMask.prec = 2;
     vMask.postTxt = qsFL1( " v" );

     break;
   case Sensor::sysCpuInfo:
   case Sensor::sysCpuLoad:

     break;
    case Sensor::sysRamUsed:
    case Sensor::sysSwapUsed:
        if ( arc )
            arc->setColorRanges(Qt::red,Qt::red,QColor(0,200,150)); // QColor(0,200,0));
     
     break;
   case Sensor::sysUpTime:

    break;
   default:

     break;
 }
 setValue( /* sensor->getValue(), sensor->getValue1(), sensor->getValue2(), sensor->getValue3(), sensor->getValue4() */ );

  if ( arc )
    arc->update();

  if ( lcdVal )
    lcdVal->update();

  if ( lcdDes )
    lcdDes->display(sensor->getDescription());

  setToolTip( sensor->getDescription() + qsFL1( " (" ) + sensor->getLongName() + qsFL1( ")" ) );
}

/***************************************************************************
    set the sensor value
 ***************************************************************************/

void SensorPanel::updateValue()
{
    if ( sensor->getValueChanged() || ( panelsWindow->getMeterTypes() & MeterTypes::Chart ) )
        setValue();
}

void SensorPanel::setValue( /* double value, double value1, double value2, double value3, double value4 */ )
{
    QString strValue1, strValue2, strValue3, strValue4, str1, str2, str3, str4;

    struct QLineChart::T_ParData parData;

    double value = sensor->getValue();
    double valueMax = sensor->getValueMax();
    double value1 = sensor->getValue1();
    double value2 = sensor->getValue2();
    double value3 = sensor->getValue3();
    double value4 = sensor->getValue4();
    QString valDescr = sensor->getValDescr();
    QString descr1 = sensor->getDescr1();
    QString descr2 = sensor->getDescr2();

    switch ( sensor->getType() )
    {
    case Sensor::sysCpuInfo:
      strValue1 = qsFL1( "%1" ).arg ( value, 0, 'f', 0, QChar::fromLatin1 ( ' ' ) );
      str1 = qsFL1( "%1" ).arg ( value, 0, 'f', 0, QChar::fromLatin1 ( ' ' ) );
      str3 = descr1;
      str2 = descr2;

      break;
    case Sensor::sysCpuLoad:
      strValue4 = QString::number( value1 );
      strValue3 = QString::number( value2 );
      strValue2 = QString::number( value3 );
      strValue1 = QString::number( value4 );
      
      str4 = qsFL1( "USER%1" ).arg ( value1, 5, 'f', 1, QChar::fromLatin1 ( ' ' ) );
      str3 = qsFL1( "NICE%1" ).arg ( value2, 5, 'f', 1, QChar::fromLatin1 ( ' ' ) );
      str2 = qsFL1( "SYS %1" ).arg ( value3, 5, 'f', 1, QChar::fromLatin1 ( ' ' ) );
      str1 = qsFL1( "IDLE%1" ).arg ( value4, 5, 'f', 1, QChar::fromLatin1 ( ' ' ) );

      descr1 = sensor->getDescription();
      
      break;
    case Sensor::sysRamUsed:
    case Sensor::sysSwapUsed:
      strValue1 = qsFL1( "%1" ).arg ( value, 0, 'f', 0, QChar::fromLatin1 ( ' ' ) );    
      str1 = qsFL1( "%1M" ).arg ( value, 0, 'f', 0, QChar::fromLatin1 ( ' ' ) );
      str3 = qsFL1( ( sensor->getType() == Sensor::sysSwapUsed ? "SWP %1M" : "RAM %1M" ) ).arg ( valueMax );
      descr1 = str3;

      break;
    case Sensor::sysUpTime:
      strValue1 = valDescr;
      str1 = valDescr;

      break;
    default:
      strValue1 = qsFL1( "%1" ).arg ( value, vMask.width, vMask.formType, vMask.prec, QChar::fromLatin1 ( ' ' ) );
      str1 = qsFL1( "%1%2%3" ).arg( vMask.preTxt ).arg ( value, vMask.width, vMask.formType, vMask.prec, QChar::fromLatin1 ( ' ' ) ).arg ( vMask.postTxt );

      descr1 = sensor->getDescription();

      break;
    }

    if ( qlChart )
    {
        parData.init( true );

        parData.qvValue1 = parData.qvDescr1 = true;
        if ( sensor->getType() == Sensor::sysCpuInfo )
            parData.qvDescr2 = true;
        else if ( sensor->getType() == Sensor::sysCpuLoad )
            parData.qvValue2 = parData.qvValue3 = parData.qvValue4 = true;

        qlChart->setValueVisible ( parData );

        parData.init( false );

        parData.enableValue1 = parData.enableDescr1 = true;
        if ( sensor->getType() == Sensor::sysCpuInfo )
            parData.enableDescr2 = true;
        else if ( sensor->getType() == Sensor::sysCpuLoad )
        {
            parData.enableValue2 = parData.enableValue3 = parData.enableValue4 = true;

            parData.qvValue1 = color1;
            parData.qvValue2 = color2;
            parData.qvValue3 = color3;
            parData.qvValue4 = color4;

            qlChart->setValueColor ( parData );
        }

        parData.qvValue1 = str1;
        parData.qvValue2 = str2;
        parData.qvValue3 = str3;
        parData.qvValue4 = str4;
        parData.qvDescr1 = descr1;
        parData.qvDescr2 = descr2;

        qlChart->setValue ( parData );

        parData.qvValue1 = strValue1;
        parData.qvValue2 = strValue2;
        parData.qvValue3 = strValue3;
        parData.qvValue4 = strValue4;

        qlChart->addValue( parData, sensor->getSensorDev()->getSensorUpdateInterval( true ) );
    }
  
  if ( lcdVal )
    lcdVal->display( str1 );

  if ( arc )
  {
    switch ( sensor->getType() )
    {
    case Sensor::sysRamUsed:
    case Sensor::sysSwapUsed:
      arc->setValueRanges( 0, valueMax );
      arc->setValueLimitRanges( value, value );

      break;
    default:

      break;
    }

    arc->setValue( value );
  }

// ptotal value, puser value1, pnice value2, psystem value3, pidle value4
  if ( barMeter )
  {    
    barMeter->setValue( 0, value1 );
    barMeter->setValue( 1, value2 );
    barMeter->setValue( 2, value3 );
    barMeter->setValue( 3, value4 );
  }

    if ( lcdVal1 )
        lcdVal1->display( ( lcdVal4 ? str4 : str1 ) );
    if ( lcdVal2 )
        lcdVal2->display( str3 );
    if ( lcdVal3 )
        lcdVal3->display( str2 );
    if ( lcdVal4 )
        lcdVal4->display( str1 );

  if ( lcdDes1 )
  {
    lcdDes1->display ( descr2 );
  }

 //
    parData.init( false );

    parData.enableValue1 = true;

 bool newAlarm= sensor->getAlarm();
    if ( newAlarm != alarm )
    {
        alarm= newAlarm;
        if ( newAlarm )
        {
            if ( !idTimer )
                idTimer = startTimer(500);

            if ( sensor->getSensorDev()->getClass() != Sensor::sysSensor )
            {
                parData.qvValue1 = QColor ( Qt::red );

                if ( qlChart )
                    qlChart->setValueColor( parData );
                else if ( lcdVal )
                    lcdVal->setForeColor( Qt::red );
            }
        }
        else
        {
            if ( idTimer )
            {
                killTimer (idTimer);
                idTimer = 0;
            }

            if ( sensor->getSensorDev()->getClass() != Sensor::sysSensor )
            {
                parData.qvValue1 = QColor ( Qt::black );

                if ( qlChart )
                    qlChart->setValueColor( parData );
                else if ( lcdVal )
                    lcdVal->setForeColor( Qt::black );
            }

            if ( qlChart )
            {
                if ( !qlChart->getValueVisible() )
                {
                    parData.qvValue1 = true;

                    qlChart->setValueVisible( parData );
                }
            }
            else if ( lcdVal )
                if(lcdVal->isHidden()) lcdVal->show();

        } // else ( newAlarm )
    } // if ( newAlarm != alarm )
}

//**********************************************************************


void SensorPanel::createTitleWidget()
{
  lcdDes = lcdDes1 = lcdDes2 = nullptr;

  switch( sensor->getType() )
  {
  case Sensor::sysCpuInfo:
  case Sensor::sysRamUsed:
  case Sensor::sysSwapUsed:
    
    break;
  case Sensor::sysUpTime:
    lcdDes = new QLCDString(this);
    lcdDes->setNumberDisplay(true);
    lcdDes->installEventFilter(this);

    lcdDes1 = new QLCDString(this);
    lcdDes1->setNumberDisplay(true);    
    lcdDes1->installEventFilter(this);
    
    break;        
  default:
    if ( !qlChart )
    {
      lcdDes = new QLCDString(this);
      lcdDes->setNumberDisplay(true);
      lcdDes->installEventFilter(this);
    }

    break;
  }
}


void SensorPanel::createGraphicWidget()
{
    arc = nullptr;
    barMeter = nullptr;

    switch( sensor->getType() )
    {
    case Sensor::sysCpuInfo:

        break;
    case Sensor::sysCpuLoad:
        if ( !qlChart )
        {
            barMeter= new QBarMeter(this);
            barMeter->setGeometry(6,6,6,40);
            barMeter->setDirection(QBarMeter::dirUp);
            barMeter->setSteps(20);
            barMeter->useValueMax(false);
            barMeter->setValueColor( 0, color4 );
            barMeter->setValueColor( 1, color3 );
            barMeter->setValueColor( 2, color2 );
            barMeter->setValueColor( 3, color1 );
            barMeter->setValueCount(4);
            barMeter->installEventFilter(this);
        }

        break;
    case Sensor::sysRamUsed:
    case Sensor::sysSwapUsed:
        if ( !qlChart )
        {
            arc = new QDialArc(this);
            arc->installEventFilter(this);
            arc->setAngles(180,-180);
        }

        break;
    case Sensor::sysUpTime:

        break;
    default:
        if ( !qlChart )
        {
            arc = new QDialArc(this);
            arc->installEventFilter(this);
            arc->setAngles(180,-180);
        }

        break;
    } // switch( sensor->getType() )
}

void SensorPanel::createValueWidget()
{
    lcdVal = lcdVal1 = lcdVal2 = lcdVal3 = lcdVal4 = nullptr;

    if (!qlChart)
    {
        switch( sensor->getType() )
        {
        case Sensor::sysCpuInfo:
            lcdVal1 = new QLCDString(this);
            lcdVal1->setGeometry(6,20,52,26);
            lcdVal1->setShadow(true);
            lcdVal1->setForeColor(Qt::red);
            lcdVal1->setShadowColor(Qt::darkRed);
            lcdVal1->installEventFilter(this);

            lcdVal2 = new QLCDString(this);    
            lcdVal2->setGeometry(6,25,52,15);
            lcdVal2->setForeColor(getColorTitle());
            lcdVal2->setAlign(QLCDString::alignJustify);
            lcdVal2->installEventFilter(this);

            lcdVal3 = new QLCDString(this);
            lcdVal3->setGeometry(4, 52,58,8);
            lcdVal3->setForeColor(getColorTitle());
            lcdVal3->setAlign(QLCDString::alignCenter);
            lcdVal3->installEventFilter(this);

            break;
        case Sensor::sysCpuLoad:
            lcdVal1= new QLCDString(this);
            lcdVal1->setGeometry(16,38,46,8);
            lcdVal1->setForeColor( color4 );
            lcdVal1->setAlign(QLCDString::alignLeft);
            lcdVal1->setNumberDisplay(true);
            lcdVal1->installEventFilter(this);

            lcdVal2= new QLCDString(this);
            lcdVal2->setGeometry(16,28,46,8);
            lcdVal2->setForeColor( color3 );
            lcdVal2->setAlign(QLCDString::alignLeft);
            lcdVal2->setNumberDisplay(true);
            lcdVal2->installEventFilter(this);

            lcdVal3= new QLCDString(this);
            lcdVal3->setGeometry(16,18,46,8);
            lcdVal3->setForeColor( color2 );
            lcdVal3->setAlign(QLCDString::alignLeft);
            lcdVal3->setNumberDisplay(true);
            lcdVal3->installEventFilter(this);

            lcdVal4= new QLCDString(this);
            lcdVal4->setGeometry(16,8,46,8);
            lcdVal4->setForeColor( color1 );
            lcdVal4->setAlign(QLCDString::alignLeft);
            lcdVal4->setNumberDisplay(true);
            lcdVal4->installEventFilter(this);

            break;
        case Sensor::sysRamUsed:
        case Sensor::sysSwapUsed:
            lcdVal1 = new QLCDString(this);
            lcdVal1->setGeometry(6, 8,54,11);
            lcdVal1->setShadow(true);
            lcdVal1->setForeColor(getColorValue());
            lcdVal1->setAlign(QLCDString::alignCenter);
            lcdVal1->installEventFilter(this);

            lcdVal2 = new QLCDString(this);
            lcdVal2->setGeometry(4, 52,58,8);
            lcdVal2->setForeColor(getColorTitle());
            lcdVal2->setAlign(QLCDString::alignCenter);
            lcdVal2->installEventFilter(this);

            break;
        case Sensor::sysUpTime:
            lcdVal1 = new QLCDString(this);
            lcdVal1->setGeometry(6,25,52,15);
            lcdVal1->setShadow(true);
            lcdVal1->setForeColor(Qt::red);
            lcdVal1->setShadowColor(Qt::darkRed);
            lcdVal1->installEventFilter(this);

            break;
        default:  
            lcdVal= new QLCDString(this);
            lcdVal->installEventFilter(this);
            lcdVal->setShadow(true);
            lcdVal->setNumberDisplay(sensor->getType()==Sensor::lmVoltage);
        
            break;
        } // switch( sensor->getType() )
    } // if (!qlChart)
}


void SensorPanel::resizeEvent ( QResizeEvent *e )
{
  Q_UNUSED(e);

 int w= width();
 int h= height();

 int i1= (h * 2) / 5;
 int i2= (h * 4) / 5;

  int lw= (w*10)/14;
  int lh= h/8;
  int sh= (h*10)/64;
  int i= h/8;
  int i0= i-1;
 
  int mw= w/10;
  int th= h-i2-h/11;

    if ( qlChart )
    {
        qlChart->setGeometry ( 0, 0, w, h );
    }

  if ( lcdVal && sensor->getType() != Sensor::sysUpTime)
  {
      if(sensor->getType()==Sensor::lmVoltage)  lcdVal->setGeometry( 3 , h/8 , w -4  , h/5-h/32);
      else                                      lcdVal->setGeometry( 3 , h/8 , w -4  , h/4-h/16);
  }

  if ( arc )
    arc->setGeometry   ( 3 , i1  , w -6 , i2-i1-2);
 
  if ( lcdVal4 )
  {    
    lcdVal4->setGeometry(w/4 ,i ,lw,lh);
    i+= sh;
  }
  if ( lcdVal3 )
  {
    lcdVal3->setGeometry(w/4 ,i ,lw,lh);
    i+= sh;
  }
  if ( lcdVal2 )
  {
    switch ( sensor->getType() )
    {
    case Sensor::sysCpuInfo:
      lcdVal2->setGeometry(0,h/10,w-0,th);
      lcdVal3->setGeometry(2,i2+1,w-4,th);

      break;
 

// QLcd::draw(p, 2,h/10,w-4,th,sCpu.toLatin1(),QLcd::alignJustify, &getColorTitle());
// QLcd::draw(p, 2,i2+1,w-4,th,sBogomips.toLatin1(),QLcd::alignCenter,&getColorTitle());

      
    case Sensor::sysRamUsed:
    case Sensor::sysSwapUsed:
      lcdVal2->setGeometry(3 , i2+1, w -6 , h-i2-h/11);

      break;
    default:
      lcdVal2->setGeometry(w/4 ,i ,lw,lh);
      i+= sh;

      break;
    }
  }
  if ( lcdVal1 )
  {
    switch ( sensor->getType() )
    {
    case Sensor::sysCpuInfo:
      lcdVal1->setGeometry(mw,h/3,w-mw*2,(h*2)/5);

      break;
    case Sensor::sysRamUsed:
    case Sensor::sysSwapUsed:
       lcdVal1->setGeometry(3 , h/8 , w -4  , h/4-h/16);

      break;
    case Sensor::sysUpTime:
      lcdVal1->setGeometry(mw,h/3,w-mw*2,(h*2)/5);

      break;
    default:
      lcdVal1->setGeometry(w/4 ,i ,lw,lh);

      break;
    }
  }
  if ( barMeter )
  {
    int bl= i+lh-i0;
    barMeter->setGeometry(w/10,i0,w/10,bl);
    barMeter->setSteps(bl/2);
  }

  if ( lcdDes )
    lcdDes->setGeometry( 3 , i2+1, w -6 , h -i2-h/11);
 
  switch ( sensor->getType() )
  {
  case Sensor::sysUpTime:
    lcdDes1->setGeometry(2,h/10,w-4,th);

    break;
  default:

    break;
  }  
}

void SensorPanel::timerEvent( QTimerEvent * )
{
    struct QLineChart::T_ParData parData;

    parData.init( false );
    
    parData.enableValue1 = true;

    if ( qlChart )
    {
        parData.qvValue1 = !qlChart->getValueVisible();
      
        qlChart->setValueVisible( parData );
    }
    else if ( lcdVal )
    {
      if  (lcdVal->isVisible()) lcdVal->hide();
      else	 			 		    	  
        lcdVal->show();
    }
  
    repaint();
}


void SensorPanel::changeEvent ( QEvent * event )
{
    struct QLineChart::T_ParData parData;

    parData.init( false );

  if ( event->type() == QEvent::PaletteChange )
  {
    if ( lcdDes )
      lcdDes->setForeColor(getColorTitle());

    if ( qlChart )
    {
        parData.enableValue1 = true;
        parData.enableDescr1 = parData.enableDescr2 = true;

        parData.qvValue1 = getColorValue();
        parData.qvDescr1 = parData.qvDescr2 = getColorTitle();

        qlChart->setValueColor( parData );
    }
    else if ( lcdVal )
    {
      lcdVal->setForeColor(getColorValue());
      lcdVal->setShadowColor(getColorValue());
    }
    if ( arc )
      arc->setArrowColor(getColorValue());

    switch ( sensor->getType() )
    {
    case Sensor::sysCpuInfo:
    case Sensor::sysCpuLoad:
    case Sensor::sysUpTime:
    
      break;
    default:
      if ( lcdVal1 )
      {
        lcdVal1->setForeColor(getColorValue());
        lcdVal1->setShadowColor(getColorValue());
      }
      if ( lcdVal2 )
      {
        lcdVal2->setForeColor(getColorValue());
        lcdVal2->setShadowColor(getColorValue());
      }
      if ( lcdVal3 )
      {
        lcdVal3->setForeColor(getColorValue());
        lcdVal3->setShadowColor(getColorValue());
      }
      if ( lcdVal4 )
      {
        lcdVal4->setForeColor(getColorValue());
        lcdVal4->setShadowColor(getColorValue());
      }
      
      break;
    } // switch ( sensor->getType() )
    if ( lcdDes1 )
    {
      lcdDes1->setForeColor(getColorTitle());
    }
    if ( lcdDes2 )
    {
      lcdDes2->setForeColor(getColorTitle());
    }  
  } // if ( event->type() == QEvent::PaletteChange )
} // void changeEvent ( QEvent * event )
